@@ -1,10 +1,10 @@
+import json
 from collections import defaultdict
 from copy import deepcopy
 from dataclasses import dataclass
-from typing import Any, Dict, List, Tuple
+from typing import Any, Dict, List, Tuple, Union
 
 from lxml import etree as ET
-from rich.pretty import pretty_repr
 
 from guardrails.utils.constants import constants
 
@@ -16,9 +16,6 @@ class ReAsk:
     fix_value: Any
     path: List[Any] = None
 
-    def __repr__(self) -> str:
-        return pretty_repr(self)
-
 
 def gather_reasks(validated_output: Dict) -> List[ReAsk]:
     """Traverse output and gather all ReAsk objects.
@@ -30,12 +27,18 @@ def gather_reasks(validated_output: Dict) -> List[ReAsk]:
     Returns:
         A list of ReAsk objects found in the output.
     """
+    from guardrails.validators import PydanticReAsk
+
     reasks = []
 
     def _gather_reasks_in_dict(output: Dict, path: List[str] = []) -> None:
+        is_pydantic = isinstance(output, PydanticReAsk)
         for field, value in output.items():
             if isinstance(value, ReAsk):
-                value.path = path + [field]
+                if is_pydantic:
+                    value.path = path
+                else:
+                    value.path = path + [field]
                 reasks.append(value)
 
             if isinstance(value, dict):
@@ -65,6 +68,12 @@ def get_reasks_by_element(
     parsed_rail: ET._Element,
 ) -> Dict[ET._Element, List[tuple]]:
     """Cluster reasks by the XML element they are associated with."""
+    # This should be guaranteed to work, since the path corresponding
+    # to a ReAsk should always be valid in the element tree.
+
+    # This is because ReAsk objects are only created for elements
+    # with corresponding validators i.e. the element must have been
+    # in the tree in the first place for the ReAsk to be created.
 
     reasks_by_element = defaultdict(list)
 
@@ -119,10 +128,17 @@ def get_pruned_tree(
                 grandparent.remove(parent)
                 parent = grandparent
 
+    pruned_elements = root.findall(".//*")
+    for element in pruned_elements:
+        if element not in reask_elements:
+            # Remove the format attribute
+            if "format" in element.attrib:
+                del element.attrib["format"]
+
     return root
 
 
-def prune_json_for_reasking(json_object: Any) -> Dict:
+def prune_json_for_reasking(json_object: Any) -> Union[None, Dict, List]:
     """Validated JSON is a nested dictionary where some keys may be ReAsk
     objects.
 
@@ -135,7 +151,11 @@ def prune_json_for_reasking(json_object: Any) -> Dict:
     Returns:
         The pruned validated JSON.
     """
-    if isinstance(json_object, list):
+    from guardrails.validators import PydanticReAsk
+
+    if isinstance(json_object, ReAsk) or isinstance(json_object, PydanticReAsk):
+        return json_object
+    elif isinstance(json_object, list):
         pruned_list = []
         for item in json_object:
             pruned_output = prune_json_for_reasking(item)
@@ -147,7 +167,7 @@ def prune_json_for_reasking(json_object: Any) -> Dict:
     elif isinstance(json_object, dict):
         pruned_json = {}
         for key, value in json_object.items():
-            if isinstance(value, ReAsk):
+            if isinstance(value, ReAsk) or isinstance(value, PydanticReAsk):
                 pruned_json[key] = value
             elif isinstance(value, dict):
                 pruned_output = prune_json_for_reasking(value)
@@ -166,37 +186,12 @@ def prune_json_for_reasking(json_object: Any) -> Dict:
             return pruned_json
 
         return None
-    else:
-        if isinstance(json_object, ReAsk):
-            return json_object
-        return None
-
-
-def extract_prompt_from_xml(tree: ET._Element) -> str:
-    """Extract the prompt from an XML tree.
-
-    Args:
-        tree: The XML tree.
-
-    Returns:
-        The prompt.
-    """
-    # From the element tree, remove any action attributes like 'on-fail-*'.
-    # Filter any elements that are comments.
-    for element in tree.iter():
-        if isinstance(element, ET._Comment):
-            continue
-
-        for attr in list(element.attrib):
-            if attr.startswith("on-fail-"):
-                del element.attrib[attr]
-
-    # Return the XML as a string.
-    return ET.tostring(tree, encoding="unicode", method="xml")
 
 
 def get_reask_prompt(
-    parsed_rail, reasks: List[ReAsk], reask_json: Dict
+    parsed_rail: ET._Element,
+    reasks: List[ReAsk],
+    reask_json: Dict,
 ) -> Tuple[str, ET._Element]:
     """Construct a prompt for reasking.
 
@@ -210,6 +205,8 @@ def get_reask_prompt(
     Returns:
         The prompt.
     """
+    from guardrails.schema import OutputSchema
+
     parsed_rail_copy = deepcopy(parsed_rail)
 
     # Get the elements that are to be reasked
@@ -218,14 +215,17 @@ def get_reask_prompt(
     # Get the pruned JSON so that it only contains ReAsk objects
     # Get the pruned tree
     pruned_tree = get_pruned_tree(parsed_rail_copy, list(reask_elements.keys()))
-    pruned_tree_string = extract_prompt_from_xml(pruned_tree)
+    pruned_tree_string = OutputSchema(pruned_tree).transpile()
 
     reask_prompt_template = (
         constants["high_level_reask_prompt"] + constants["complete_json_suffix"]
     )
 
+    def reask_decoder(obj):
+        return {k: v for k, v in obj.__dict__.items() if k not in ["path", "fix_value"]}
+
     reask_prompt = reask_prompt_template.format(
-        previous_response=pretty_repr(reask_json),
+        previous_response=json.dumps(reask_json, indent=2, default=reask_decoder),
         output_schema=pruned_tree_string,
     )
 
