@@ -32,6 +32,10 @@ types_to_validators = defaultdict(list)
 logger = logging.getLogger(__name__)
 
 
+class ValidatorError(Exception):
+    """Base class for all validator errors."""
+
+
 class Filter:
     pass
 
@@ -255,11 +259,45 @@ class Validator:
 
         return error.schema
 
+    def exception(self, error: EventDetail) -> None:
+        """Raise an exception."""
+
+        raise ValidatorError(error.error_message)
+
+    def fix_reask(self, error: EventDetail) -> Dict:
+        """If validation fails, fix the value and reask."""
+
+        schema = self.fix(error)
+
+        try:
+            self.validate(error.key, error.fix_value, schema)
+        except EventDetail as e:
+            return self.reask(e)
+
     def to_prompt(self, with_keywords: bool = True) -> str:
-        """Convert the validator to a prompt."""
-        params = " ".join(list(self._kwargs.values()))
+        """Convert the validator to a prompt.
+
+        E.g. ValidLength(5, 10) -> "length: 5 10" when with_keywords is False.
+        ValidLength(5, 10) -> "length: min=5 max=10" when with_keywords is True.
+
+        Args:
+            with_keywords: Whether to include the keyword arguments in the prompt.
+
+        Returns:
+            A string representation of the validator.
+        """
+
+        if not len(self._kwargs):
+            return self.rail_alias
+
+        kwargs = self._kwargs.copy()
+        for k, v in kwargs.items():
+            if not isinstance(v, str):
+                kwargs[k] = str(v)
+
+        params = " ".join(list(kwargs.values()))
         if with_keywords:
-            params = " ".join([f"{k}={v}" for k, v in self._kwargs.items()])
+            params = " ".join([f"{k}={v}" for k, v in kwargs.items()])
         return f"{self.rail_alias}: {params}"
 
 
@@ -367,6 +405,67 @@ class Pydantic(Validator):
         return schema
 
 
+@register_validator(name="choice", data_type="choice")
+class Choice(Validator):
+    """Validate that a value is one of a set of choices.
+
+    - Name for `format` attribute: `choice`
+    - Supported data types: `string`
+    - Programmatic fix: Closest value within the set of choices.
+    """
+
+    def __init__(
+        self,
+        choices: List[str],
+        on_fail: Optional[Callable] = None,
+    ):
+        super().__init__(on_fail=on_fail, choices=choices)
+
+        self._choices = choices
+
+    def validate(self, key: str, value: Any, schema: Union[Dict, List]) -> Dict:
+        """Validate that a value is one of a set of choices."""
+
+        logger.debug(f"Validating {value} is in {self._choices}...")
+
+        if value not in self._choices:
+            raise EventDetail(
+                key=key,
+                value=value,
+                schema=schema,
+                error_message=f"{value} is not in {self._choices}",
+                fix_value=None,
+            )
+
+        selected_choice = value
+        if selected_choice not in schema:
+            raise EventDetail(
+                key=key,
+                value=value,
+                schema=schema,
+                error_message=f"{schema} must contain a key called {value}",
+                fix_value=None,
+            )
+
+        # Make sure that no other choice is selected.
+        for choice in self._choices:
+            if choice == selected_choice:
+                continue
+            if choice in schema:
+                raise EventDetail(
+                    key=key,
+                    value=value,
+                    schema=schema,
+                    error_message=(
+                        f"{schema} must not contain a key called {choice}, "
+                        f"since {selected_choice} is selected"
+                    ),
+                    fix_value=None,
+                )
+
+        return schema
+
+
 @register_validator(name="valid-range", data_type=["integer", "float", "percentage"])
 class ValidRange(Validator):
     """Validate that a value is within a range.
@@ -379,7 +478,7 @@ class ValidRange(Validator):
     def __init__(
         self, min: int = None, max: int = None, on_fail: Optional[Callable] = None
     ):
-        super().__init__(on_fail=on_fail)
+        super().__init__(on_fail=on_fail, min=min, max=max)
 
         self._min = min
         self._max = max
@@ -422,7 +521,7 @@ class ValidChoices(Validator):
     """
 
     def __init__(self, choices: List[Any], on_fail: Optional[Callable] = None):
-        super().__init__(on_fail=on_fail)
+        super().__init__(on_fail=on_fail, choices=choices)
         self._choices = choices
 
     def validate(self, key: str, value: Any, schema: Union[Dict, List]) -> Dict:
@@ -490,7 +589,7 @@ class UpperCase(Validator):
         return schema
 
 
-@register_validator(name="length", data_type=["string", "list", "object"])
+@register_validator(name="length", data_type=["string", "list"])
 class ValidLength(Validator):
     """Validate that the length of value is within the expected range.
 
@@ -503,7 +602,7 @@ class ValidLength(Validator):
     def __init__(
         self, min: int = None, max: int = None, on_fail: Optional[Callable] = None
     ):
-        super().__init__(on_fail=on_fail)
+        super().__init__(on_fail=on_fail, min=min, max=max)
         self._min = int(min) if min is not None else None
         self._max = int(max) if max is not None else None
 
@@ -518,7 +617,12 @@ class ValidLength(Validator):
             logger.debug(f"Value {value} is less than {self._min}.")
 
             # Repeat the last character to make the value the correct length.
-            corrected_value = value + value[-1] * (self._min - len(value))
+            if isinstance(value, str):
+                last_val = value[-1]
+            else:
+                last_val = [value[-1]]
+
+            corrected_value = value + last_val * (self._min - len(value))
             raise EventDetail(
                 key,
                 value,
@@ -850,7 +954,7 @@ class EndsWith(Validator):
     """
 
     def __init__(self, end: str, on_fail: str = "fix"):
-        super().__init__(on_fail=on_fail)
+        super().__init__(on_fail=on_fail, end=end)
         self._end = end
 
     def validate(self, key: str, value: Any, schema: Union[Dict, List]) -> Dict:
